from ..models import District

legacy_districts = {
    "md": [
        District("12A", "lower", division_id=None),
        District("12B", "lower", division_id=None),
        District("2C", "lower", division_id=None),
        District("30", "lower", division_id=None),
        District("31", "lower", division_id=None),
        District("33A", "lower", division_id=None),
        District("33B", "lower", division_id=None),
        District("42", "lower", division_id=None),
        District("44", "lower", division_id=None),
        District("47", "lower", division_id=None),
        District("4A", "lower", division_id=None),
        District("4B", "lower", division_id=None),
        District("5A", "lower", division_id=None),
        District("5B", "lower", division_id=None),
    ],
    "nv": [
        District("Capital Senatorial District", "upper", division_id=None),
        District("Central Nevada Senatorial District", "upper", division_id=None),
        District("Clark County, No. 1", "upper", division_id=None),
        District("Clark County, No. 3", "upper", division_id=None),
        District("Clark County, No. 4", "upper", division_id=None),
        District("Clark County, No. 5", "upper", division_id=None),
        District("Clark County, No. 6", "upper", division_id=None),
        District("Clark County, No. 7", "upper", division_id=None),
        District("Clark County, No. 8", "upper", division_id=None),
        District("Clark County, No. 9", "upper", division_id=None),
        District("Clark County, No. 9", "upper", division_id=None),
        District("Clark County, No. 10", "upper", division_id=None),
        District("Clark County, No. 11", "upper", division_id=None),
        District("Clark County, No. 12", "upper", division_id=None),
        District("Rural Nevada Senatorial District", "upper", division_id=None),
        District("Washoe County, No. 1", "upper", division_id=None),
        District("Washoe County, No. 2", "upper", division_id=None),
        District("Washoe County, No. 3", "upper", division_id=None),
        District("Washoe County, No. 4", "upper", division_id=None),
    ],
    "pr": [District(str(n), "upper", division_id=None) for n in range(1, 9)],
    "vt": [
        District("Addison-Rutland-1", "lower", division_id=None),
        District("Bennington-Rutland-1", "lower", division_id=None),
        District("Chittenden-1-2", "lower", division_id=None),
        District("Chittenden-3-1", "lower", division_id=None),
        District("Chittenden-3-2", "lower", division_id=None),
        District("Chittenden-3-3", "lower", division_id=None),
        District("Chittenden-3-4", "lower", division_id=None),
        District("Chittenden-3-5", "lower", division_id=None),
        District("Chittenden-3-6", "lower", division_id=None),
        District("Chittenden-3-7", "lower", division_id=None),
        District("Chittenden-3-9", "lower", division_id=None),
        District("Chittenden-4", "lower", division_id=None),
        District("Chittenden-8", "lower", division_id=None),
        District("Chittenden-9", "lower", division_id=None),
        District("Franklin-3", "lower", division_id=None),
        District("Grand Isle-Chittenden-1-1", "lower", division_id=None),
        District("Lamoille-4", "lower", division_id=None),
        District("Lamoille-Washington-1", "lower", division_id=None),
        District("Orange-Addison-1", "lower", division_id=None),
        District("Orleans-Caledonia-1", "lower", division_id=None),
        District("Rutland-1-1", "lower", division_id=None),
        District("Rutland-1-2", "lower", division_id=None),
        District("Rutland-7", "lower", division_id=None),
        District("Rutland-8", "lower", division_id=None),
        District("Washington-3-2", "lower", division_id=None),
        District("Washington-3-3", "lower", division_id=None),
        District("Washington-Chittenden-1", "lower", division_id=None),
        District("Windham-2", "lower", division_id=None),
        District("Windham-3-1", "lower", division_id=None),
        District("Windham-3-3", "lower", division_id=None),
        District("Windham-Bennington-1", "lower", division_id=None),
        District("Windham-Bennington-Windsor-1", "lower", division_id=None),
        District("Windsor-1-1", "lower", division_id=None),
        District("Windsor-1-2", "lower", division_id=None),
        District("Windsor-3", "lower", division_id=None),
        District("Windsor-4", "lower", division_id=None),
        District("Windsor-6-1", "lower", division_id=None),
        District("Windsor-6-2", "lower", division_id=None),
    ],
    "us": [
        District("IL-20", "lower", division_id=None),
        District("MA-10", "lower", division_id=None),
        District("MO-9", "lower", division_id=None),
        District("NJ-13", "lower", division_id=None),
        District("NY-29", "lower", division_id=None),
        District("OH-17", "lower", division_id=None),
        District("OH-18", "lower", division_id=None),
        District("OK-6", "lower", division_id=None),
    ],
}
